package com.reactlibraryneurosdk;

import android.os.Build;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.neuromd.neurosdk.BaseChannel;
import com.neuromd.neurosdk.ChannelInfo;
import com.neuromd.neurosdk.Command;
import com.neuromd.neurosdk.Device;
import com.neuromd.neurosdk.DeviceState;
import com.neuromd.neurosdk.FirmwareVersion;
import com.neuromd.neurosdk.Parameter;
import com.neuromd.neurosdk.ParameterName;

import java.util.HashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class BrainBit {

    private Device device;
    private final ReactApplicationContext reactContext;

    /* Channels */
    private final HashMap<String, SignalChannel> _channelsSignal = new HashMap<>();
    private final HashMap<String, BatteryChannel> _channelsBattery = new HashMap<>();
    private final HashMap<String, ResistanceChannel> _channelsResistance = new HashMap<>();

    /*Threads*/
    ExecutorService threadService;

    public BrainBit(@NonNull Device device, @NonNull ReactApplicationContext reactContext) {
        this.device = device;
        this.reactContext = reactContext;

        threadService = Executors.newCachedThreadPool();
    }

    public Device getDevice() {
        return device;
    }
    public void Connect(Promise promise)
    {
        try{
            threadService.submit(new Runnable() {
                @Override
                public void run() {
                    try {
                        assert device != null;
                        device.connect();
                        WritableMap resultState = Arguments.createMap();
                        resultState.putString("state", "Connection");
                        promise.resolve(resultState);
                    } catch (Exception ex) {
                        promise.reject(ex);
                    }
                }
            });
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }

    public void SubscribeParameterChanged(Promise promise){
        try{
            assert device != null;
            device.parameterChanged.subscribe(
                    (o, parameterName) -> {
                        DeviceState state = device.readParam(ParameterName.State);
                        WritableMap currentState = Arguments.createMap();
                        currentState.putString("address", device.readParam(ParameterName.Address));
                        currentState.putString("state", state.toString());
                        Utils.sendEvent(reactContext, RNNeurosdkModule.DEVICE_PARAMETER_CHANGED_EVENT_NAME, currentState);
                    }
            );
            WritableMap resultState = Arguments.createMap();
            resultState.putString("subscription", "successful");
            promise.resolve(resultState);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }

    public void UnsubscribeParameterChanged(Promise promise){
        try{
            assert device != null;
            device.parameterChanged.unsubscribe();
            WritableMap resultState = Arguments.createMap();
            resultState.putString("unsubscribe", "successful");
            promise.resolve(resultState);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }

    public void FirmwareVersion(Promise promise){
        try {
            assert device != null;
            FirmwareVersion fVersion = device.readParam(ParameterName.FirmwareVersion);
            WritableMap outVersion = Arguments.createMap();
            outVersion.putString("version", String.valueOf(fVersion.version()));
            outVersion.putString("build", String.valueOf(fVersion.build()));
            promise.resolve(outVersion);
        } catch (Exception ex) {
            promise.reject(ex);
        }
    }

    public void Execute(String command, Promise promise){
        try{
            threadService.submit(new Runnable() {
                @Override
                public void run() {
                    try{
                        switch (command) {
                            case "StartSignal":
                                device.execute(Command.StartSignal);
                                break;
                            case "StopSignal":
                                device.execute(Command.StopSignal);
                                break;
                            case "StartResist":
                                device.execute(Command.StartResist);
                                break;
                            case "StopResist":
                                device.execute(Command.StopResist);
                                break;
                        }
                        promise.resolve("Command executed");
                    }
                    catch(Exception ex){
                        promise.reject(ex);
                    }
                }
            });
        }
        catch(Exception ex){
            promise.reject(ex);
        }


    }

    public void Channels(Promise promise){
        try{
            WritableArray channels = Arguments.createArray();
            for(int i = 0; i < device.channels().length; i++){
                ChannelInfo chInfo = device.channels()[i];
                WritableMap channel = Arguments.createMap();
                channel.putString("name", chInfo.getName());
                channel.putInt("index", (int)chInfo.getIndex());
                channel.putString("type", chInfo.getType().toString());
                channels.pushMap(channel);
            }
            promise.resolve(channels);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }

    public void Commands(Promise promise){
        try{
            WritableArray commands = Arguments.createArray();
            for(int i = 0; i < device.commands().length; i++){
                Command command = device.commands()[i];
                WritableMap commandMap = Arguments.createMap();
                commandMap.putString("command", command.toString());
                commands.pushMap(commandMap);
            }
            promise.resolve(commands);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }

    public void Parameters(Promise promise){
        try{
            WritableArray parameters = Arguments.createArray();
            for(int i = 0; i < device.parameters().length; i++){
                Parameter param = device.parameters()[i];
                WritableMap paramMap = Arguments.createMap();
                paramMap.putString("name", param.getName().toString());
                paramMap.putString("access", param.getAccess().toString());
                paramMap.putString("type", param.getType().toString());
                parameters.pushMap(paramMap);
            }
            promise.resolve(parameters);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }

    public void Disconnect(Promise promise){
        try{
            device.disconnect();
            WritableMap resultState = Arguments.createMap();
            resultState.putString("state", "Disconnection");
            promise.resolve(resultState);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }

    public void Close(Promise promise){
        try{
            if(device.parameterChanged != null){
                device.parameterChanged.unsubscribe();
            }
            device.disconnect();
            device.close();
            device = null;

            threadService.shutdown();

            WritableMap resultState = Arguments.createMap();
            resultState.putString("state", "Closed");
            promise.resolve(resultState);
        }
        catch(Exception ex){
            promise.reject(ex);
        }
    }
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /* Channels */
    public void CreateChannel(ChannelInfo channelInfo, Promise promise) {
        switch (channelInfo.getType()) {
            case Battery:
                BatteryChannel channelBattery = new BatteryChannel(reactContext,device,promise);
                _channelsBattery.put(channelInfo.getName(),channelBattery);
                break;
            case Signal:
                SignalChannel channelSignal = new SignalChannel(reactContext,channelInfo,device,promise);
                _channelsSignal.put(channelInfo.getName(),channelSignal);
                break;
            case Resistance:
                ResistanceChannel channelResistance = new ResistanceChannel(reactContext,channelInfo, device, promise);
                _channelsResistance.put(channelInfo.getName(),channelResistance);
                break;
            default:
                promise.reject("CreateChannel Error", "Channel type not support");
                return;
        }
    }

    public void TotalLengthChannel(ChannelInfo channelInfo,Promise promise){
        try{
            threadService.submit(new Runnable() {
                @Override
                public void run() {
                    switch (channelInfo.getType()) {
                        case Battery:
                            BatteryChannel channelBattery = _channelsBattery.get(channelInfo.getName());
                            if(channelBattery != null) channelBattery.TotalLength(promise);
                            else promise.reject("TotalLengthChannel","Channel is null");
                            break;
                        case Signal:
                            SignalChannel channelSignal = _channelsSignal.get(channelInfo.getName());
                            if(channelSignal != null) channelSignal.TotalLength(promise);
                            else promise.reject("TotalLengthChannel","Channel is null");
                            break;
                        case Resistance:
                            ResistanceChannel channelResistance = _channelsResistance.get(channelInfo.getName());
                            if(channelResistance != null) channelResistance.TotalLength(promise);
                            else promise.reject("TotalLengthChannel","Channel is null");
                            break;
                        default:
                            promise.reject("TotalLengthChannel", "Channel type not support");
                            return;
                    }
                }
            });
        }
        catch(Exception ex){
            promise.reject("TotalLengthChannel", ex);
        }
    }

    public WritableMap TotalLengthSynchronousChannel(ChannelInfo channelInfo){
        try{
            WritableMap result = Arguments.createMap();
            result.putInt("totalLength", -1);
            switch (channelInfo.getType()) {
                case Battery:
                    BatteryChannel channelBattery = _channelsBattery.get(channelInfo.getName());
                    if(channelBattery != null) return channelBattery.TotalLengthSynchronous();
                    else return result;
                case Signal:
                    SignalChannel channelSignal = _channelsSignal.get(channelInfo.getName());
                    if(channelSignal != null) return channelSignal.TotalLengthSynchronous();
                    else return result;
                case Resistance:
                    ResistanceChannel channelResistance = _channelsResistance.get(channelInfo.getName());
                    if(channelResistance != null) return channelResistance.TotalLengthSynchronous();
                    else return result;
                default:
                    return result;
            }
        }
        catch(Exception ex){
            //promise.reject("TotalLengthChannel", ex);
        }
        return Arguments.createMap();
    }

    public void ReadDataChannel(ChannelInfo channelInfo, int offset, int length, Promise promise){
        try{
            threadService.submit(new Runnable() {
                @Override
                public void run() {
                    switch (channelInfo.getType()) {
                        case Battery:
                            BatteryChannel channelBattery = _channelsBattery.get(channelInfo.getName());
                            if(channelBattery != null) channelBattery.ReadData(offset,length,promise);
                            else promise.reject("ReadDataChannel","Channel is null");
                        case Signal:
                            SignalChannel channelSignal = _channelsSignal.get(channelInfo.getName());
                            if(channelSignal != null) channelSignal.ReadData(offset,length,promise);
                            else promise.reject("ReadDataChannel","Channel is null");
                        case Resistance:
                            ResistanceChannel channelResistance = _channelsResistance.get(channelInfo.getName());
                            if(channelResistance != null) channelResistance.ReadData(offset,length,promise);
                            else promise.reject("ReadDataChannel","Channel is null");
                        default:
                            promise.reject("ReadDataChannel", "Channel type not support");
                            return;
                    }
                }
            });
        }
        catch(Exception ex){
            promise.reject("ReadDataChannel", ex);
        }

    }
    public WritableArray ReadDataSynchronousChannel(ChannelInfo channelInfo, int offset, int length){
        switch (channelInfo.getType()) {
            case Battery:
                BatteryChannel channelBattery = _channelsBattery.get(channelInfo.getName());
                if(channelBattery != null) return channelBattery.ReadDataSynchronous(offset,length);
                else return null;
            case Signal:
                SignalChannel channelSignal = _channelsSignal.get(channelInfo.getName());
                if(channelSignal != null) return channelSignal.ReadDataSynchronous(offset,length);
                else return null;
            case Resistance:
                ResistanceChannel channelResistance = _channelsResistance.get(channelInfo.getName());
                if(channelResistance != null) return channelResistance.ReadDataSynchronous(offset,length);
                else return null;
            default:
                return null;
        }
    }

    public void CloseChannel(ChannelInfo channelInfo, Promise promise){
        switch (channelInfo.getType()) {
            case Battery:
                BatteryChannel channelBattery = _channelsBattery.get(channelInfo.getName());
                if(channelBattery != null) channelBattery.CloseChannel(promise);
                else promise.reject("CloseChannel","Channel is null");
            case Signal:
                SignalChannel channelSignal = _channelsSignal.get(channelInfo.getName());
                if(channelSignal != null) channelSignal.CloseChannel(promise);
                else promise.reject("CloseChannel","Channel is null");
            case Resistance:
                ResistanceChannel channelResistance = _channelsResistance.get(channelInfo.getName());
                if(channelResistance != null) channelResistance.CloseChannel(promise);
                else promise.reject("CloseChannel","Channel is null");
            default:
                promise.reject("CloseChannel", "Channel type not support");
                return;
        }
    }
}
