
package com.reactlibraryneurosdk;

import android.Manifest;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.neuromd.neurosdk.ChannelInfo;
import com.neuromd.neurosdk.ChannelType;
import com.neuromd.neurosdk.Device;
import com.neuromd.neurosdk.DeviceEnumerator;
import com.neuromd.neurosdk.DeviceInfo;
import com.neuromd.neurosdk.DeviceType;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public class RNNeurosdkModule extends ReactContextBaseJavaModule {

    private final ReactApplicationContext reactContext;

    private DeviceEnumerator _scanner;
    private BroadcastReceiver receiverBond;

    /*Constants*/
    static final String DEVICE_LIST_CHANGED_EVENT_NAME = "DeviceListChanged";
    static final String DEVICE_PARAMETER_CHANGED_EVENT_NAME = "DeviceParameterChanged";
    static final String SIGNAL_CHANNEL_LENGTH_CHANGED_EVENT_NAME = "SignalChannelLengthChanged";
    static final String RESISTANCE_CHANNEL_LENGTH_CHANGED_EVENT_NAME = "ResistanceChannelLengthChanged";
    static final String BATTERY_CHANNEL_LENGTH_CHANGED_EVENT_NAME = "BatteryChannelLengthChanged";

    static final String BATTERY_CHANNEL = "Battery";
    static final String SIGNAL_CHANNEL = "Signal";
    static final String RESISTANCE_CHANNEL = "Resistance";

    static final String START_SIGNAL = "StartSignal";
    static final String STOP_SIGNAL = "StopSignal";
    static final String START_RESIST = "StartResist";
    static final String STOP_RESIST = "StopResist";
    /////////////

    public RNNeurosdkModule(ReactApplicationContext reactContext) {
        super(reactContext);
        this.reactContext = reactContext;
    }

    @Override
    public String getName() {
        return "RNNeurosdk";
    }

    private void sendEvent(ReactContext reactContext, String eventName, @Nullable WritableMap params) {
        reactContext.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class).emit(eventName, params);
    }

    @Override
    public Map<String, Object> getConstants() {
        final Map<String, Object> constants = new HashMap<>();
        constants.put("DEVICE_LIST_CHANGED_EVENT_NAME", DEVICE_LIST_CHANGED_EVENT_NAME);
        constants.put("DEVICE_PARAMETER_CHANGED_EVENT_NAME", DEVICE_PARAMETER_CHANGED_EVENT_NAME);
        constants.put("SIGNAL_CHANNEL_LENGTH_CHANGED_EVENT_NAME", SIGNAL_CHANNEL_LENGTH_CHANGED_EVENT_NAME);
        constants.put("RESISTANCE_CHANNEL_LENGTH_CHANGED_EVENT_NAME", RESISTANCE_CHANNEL_LENGTH_CHANGED_EVENT_NAME);
        constants.put("BATTERY_CHANNEL_LENGTH_CHANGED_EVENT_NAME", BATTERY_CHANNEL_LENGTH_CHANGED_EVENT_NAME);
        constants.put("BATTERY_CHANNEL", BATTERY_CHANNEL);
        constants.put("SIGNAL_CHANNEL", SIGNAL_CHANNEL);
        constants.put("RESISTANCE_CHANNEL", RESISTANCE_CHANNEL);
        constants.put("START_SIGNAL", START_SIGNAL);
        constants.put("STOP_SIGNAL", STOP_SIGNAL);
        constants.put("START_RESIST", START_RESIST);
        constants.put("STOP_RESIST", STOP_RESIST);
        return constants;
    }

    @ReactMethod
    public void addListener(String eventName) {
        // Set up any upstream listeners or background tasks as necessary
    }

    @ReactMethod
    public void removeListeners(Integer count) {
        // Remove upstream listeners, stop unnecessary background tasks
    }

    @ReactMethod
    public void CreateDeviceEnumerator(Promise promise) {
        if (_scanner == null) {
            Context context = getReactApplicationContext();
            _scanner = new DeviceEnumerator(context, DeviceType.BrainbitAny);
            _scanner.deviceListChanged.subscribe((sender, param) -> {
                sendEvent(reactContext, DEVICE_LIST_CHANGED_EVENT_NAME, null);
            });
            promise.resolve("DeviceEnumerator created");
        } else {
            promise.reject("DeviceEnumerator ", "DeviceEnumerator already exists");
        }
    }

    @ReactMethod
    public void Devices(Promise promise) {
        WritableArray devicesList = Arguments.createArray();
        for (DeviceInfo deviceInfo : _scanner.devices()) {
            WritableMap device = Arguments.createMap();
            device.putString("name", deviceInfo.name());
            device.putString("address", deviceInfo.address());
            device.putDouble("serialNumber", deviceInfo.serialNumber());
            devicesList.pushMap(device);
        }
        promise.resolve(devicesList);
    }

    @ReactMethod
    public void CreateDevice(ReadableMap deviceInfo, Promise promise) {
        if (deviceInfo.getString("address") != null) {
            try {
                DeviceInfo info = new DeviceInfo(deviceInfo.getString("name"), deviceInfo.getString("address"), (long) deviceInfo.getInt("serialNumber"));
                Device device = _scanner.createDevice(info);
                BrainBit brainBit = new BrainBit(device, reactContext);
                _deviceList.put(info.address(), brainBit);
                promise.resolve("Device created");
            } catch (Exception e) {
                promise.reject("Device created error", e);
                e.printStackTrace();
            }
        } else {
            promise.reject("Device created: ", "Device info null");
        }
    }

    @ReactMethod
    public void DeleteDeviceEnumerator() {
        if (_scanner != null) {
            try {
                _scanner.deviceListChanged.unsubscribe();
                _scanner.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
            _scanner = null;
        }
    }

    /* Device */

    private final HashMap<String, BrainBit> _deviceList = new HashMap<>();

    @ReactMethod
    public void Connect(ReadableMap deviceInfo, Promise promise) {
        BrainBit device = _deviceList.get(deviceInfo.getString("address"));
        if (device == null) {
            promise.reject("Connect command:", "Device is null");
            return;
        }
        device.Connect(promise);
    }

    @ReactMethod
    public void SubscribeParameterChanged(ReadableMap deviceInfo, Promise promise){
        BrainBit device = _deviceList.get(deviceInfo.getString("address"));
        if (device == null) {
            promise.reject("Subscribe parameter changed:", "Device is null");
            return;
        }
        device.SubscribeParameterChanged(promise);
    }

    @ReactMethod
    public void UnsubscribeParameterChanged(ReadableMap deviceInfo, Promise promise){
        BrainBit device = _deviceList.get(deviceInfo.getString("address"));
        if (device == null) {
            promise.reject("Unsubscribe parameter changed:", "Device is null");
            return;
        }
        device.UnsubscribeParameterChanged(promise);
    }

    @ReactMethod
    public void FirmwareVersion(ReadableMap deviceInfo, Promise promise){
        BrainBit device = _deviceList.get(deviceInfo.getString("address"));
        if (device == null) {
            promise.reject("Read firmware version:", "Device is null");
            return;
        }
        device.FirmwareVersion(promise);
    }

    @ReactMethod
    public void Execute(ReadableMap deviceInfo, String command, Promise promise) {
        BrainBit device = _deviceList.get(deviceInfo.getString("address"));
        if (device == null) {
            promise.reject("Execute command:", "Device is null");
            return;
        }
        device.Execute(command, promise);
    }

    @ReactMethod
    public void Channels(ReadableMap deviceInfo, Promise promise) {
        BrainBit device = _deviceList.get(deviceInfo.getString("address"));
        if (device == null) {
            promise.reject("Channels command:", "Device is null");
            return;
        }
        device.Channels(promise);
    }

    @ReactMethod
    public void Commands(ReadableMap deviceInfo, Promise promise) {
        BrainBit device = _deviceList.get(deviceInfo.getString("address"));
        if (device == null) {
            promise.reject("Commands command:", "Device is null");
            return;
        }
        device.Commands(promise);
    }

    @ReactMethod
    public void Parameters(ReadableMap deviceInfo, Promise promise) {
        BrainBit device = _deviceList.get(deviceInfo.getString("address"));
        if (device == null) {
            promise.reject("Parameters command:", "Device is null");
            return;
        }
        device.Parameters(promise);
    }

    @ReactMethod
    public void Disconnect(ReadableMap deviceInfo, Promise promise) {
        BrainBit device = _deviceList.get(deviceInfo.getString("address"));
        if (device == null) {
            promise.reject("Disconnect command:", "Device is null");
            return;
        }
        device.Disconnect(promise);
    }

    @ReactMethod
    public void Close(ReadableMap deviceInfo, Promise promise) {
        BrainBit device = _deviceList.get(deviceInfo.getString("address"));
        if (device == null) {
            promise.reject("Close command:", "Device is null");
            return;
        }
        device.Close(promise);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////

    /* Channels */

    @ReactMethod
    public void CreateChannel(ReadableMap deviceInfo, ReadableMap channelInfo, Promise promise) {
        try {
            BrainBit device = _deviceList.get(deviceInfo.getString("address"));
            if (device == null) {
                promise.reject("Error:", "Device is null");
                return;
            }
            ChannelInfo chInfo = new ChannelInfo(
                    ChannelType.valueOf(channelInfo.getString("type")),
                    channelInfo.getString("name"),
                    (long) channelInfo.getInt("index")
            );
            device.CreateChannel(chInfo, promise);
        } catch (Exception ex) {
            promise.reject(ex);
        }
    }

    @ReactMethod
    public void TotalLengthChannel(ReadableMap deviceInfo, ReadableMap channelInfo, Promise promise) {
        try {
            BrainBit device = _deviceList.get(deviceInfo.getString("address"));
            if (device == null) {
                promise.reject("Error:", "Device is null");
                return;
            }
            ChannelInfo chInfo = new ChannelInfo(
                    ChannelType.valueOf(channelInfo.getString("type")),
                    channelInfo.getString("name"),
                    (long) channelInfo.getInt("index")
            );
            device.TotalLengthChannel(chInfo, promise);
        } catch (Exception ex) {
            promise.reject(ex);
        }
    }

    @ReactMethod(isBlockingSynchronousMethod = true)
    public WritableMap TotalLengthSyncChannel(ReadableMap deviceInfo, ReadableMap channelInfo) {
        WritableMap result = Arguments.createMap();
        result.putInt("totalLength", -1);
        try {
            BrainBit device = _deviceList.get(deviceInfo.getString("address"));
            if (device == null) {
                return result;
            }
            ChannelInfo chInfo = new ChannelInfo(
                    ChannelType.valueOf(channelInfo.getString("type")),
                    channelInfo.getString("name"),
                    (long) channelInfo.getInt("index")
            );
            return device.TotalLengthSynchronousChannel(chInfo);
        }
        catch (Exception ex) {}
        finally {
            return result;
        }
    }

    @ReactMethod
    public void ReadDataChannel(ReadableMap deviceInfo, ReadableMap channelInfo, int offset, int length, Promise promise) {
        try {
            BrainBit device = _deviceList.get(deviceInfo.getString("address"));
            if (device == null) {
                promise.reject("Error:", "Device is null");
                return;
            }
            ChannelInfo chInfo = new ChannelInfo(
                    ChannelType.valueOf(channelInfo.getString("type")),
                    channelInfo.getString("name"),
                    (long) channelInfo.getInt("index")
            );
            device.ReadDataChannel(chInfo, offset, length, promise);
        } catch (Exception ex) {
            promise.reject(ex);
        }
    }

    @ReactMethod(isBlockingSynchronousMethod = true)
    public WritableArray ReadDataSyncChannel(ReadableMap deviceInfo, ReadableMap channelInfo, int offset, int length) {
        BrainBit device = _deviceList.get(deviceInfo.getString("address"));
        if (device == null) {
            return null;
        }
        ChannelInfo chInfo = new ChannelInfo(
                ChannelType.valueOf(channelInfo.getString("type")),
                channelInfo.getString("name"),
                (long) channelInfo.getInt("index")
        );
        return device.ReadDataSynchronousChannel(chInfo, offset, length);
    }

    @ReactMethod
    public void CloseChannel(ReadableMap deviceInfo, ReadableMap channelInfo, Promise promise) {
        try {
            BrainBit device = _deviceList.get(deviceInfo.getString("address"));
            if (device == null) {
                promise.reject("Error:", "Device is null");
                return;
            }
            ChannelInfo chInfo = new ChannelInfo(
                    ChannelType.valueOf(channelInfo.getString("type")),
                    channelInfo.getString("name"),
                    (long) channelInfo.getInt("index")
            );
            device.CloseChannel(chInfo, promise);
        } catch (Exception ex) {
            promise.reject(ex);
        }
    }

    @ReactMethod
    private void BondTo(String devAddress, Promise promise) {
        BluetoothAdapter bAdapter = BluetoothAdapter.getDefaultAdapter();
        if (ActivityCompat.checkSelfPermission(this.reactContext, Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED) {

        }
        Set<BluetoothDevice> devBonded = bAdapter.getBondedDevices();
        boolean ready = false;
        for (BluetoothDevice it : devBonded) {
            if (it.getAddress().equals(devAddress)) {
                ready = true;
                break;
            }
        }
        if (!ready) {
            BluetoothDevice dev = bAdapter.getRemoteDevice(devAddress);
            if (dev != null) {
                receiverBond = new BroadcastReceiver() {
                    @Override
                    public void onReceive(Context context, Intent intent) {
                        if (intent != null) {
                            Bundle extras = intent.getExtras();
                            if (extras != null) {
                                int state = extras.getInt(BluetoothDevice.EXTRA_BOND_STATE);
                                if (state == BluetoothDevice.BOND_BONDING) {
                                    return;
                                } else if (state == BluetoothDevice.BOND_BONDED) {
                                    promise.resolve(true);
                                } else if (state == BluetoothDevice.BOND_NONE) {
                                    promise.reject("bondTo", "bond fail");
                                }
                                reactContext.unregisterReceiver(receiverBond);
                            }
                        }
                    }
                };
                reactContext.registerReceiver(receiverBond, new IntentFilter(BluetoothDevice.ACTION_BOND_STATE_CHANGED));
                dev.createBond();
            }
        } else {
            promise.resolve(ready);
        }
    }
}