//
//  BrainBit.m
//  RNNeurosdk
//
//  Created by Aseatari on 23.05.2022.
//  Copyright © 2022 Facebook. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "BrainBit.h"

@implementation BrainBit

- (instancetype)init {
  self = [super init];
  if (self) {
      _resistanceChannels = [NSMutableDictionary new];
      _signalChannels = [NSMutableDictionary new];
  }
  return self;
}

- (void) createDevice: (NTDeviceEnumerator*)deviceEnumerator deviceInfo:(NTDeviceInfo*)devInfo{
    _device = [[NTDevice alloc] initWithEnumerator:deviceEnumerator deviceInfo:devInfo];
}

- (void) connect {
    [_device connect];
}

- (void) subscribeToParameterChanged{
    [_device subscribeParameterChangedWithSubscriber:^(NTParameter param){
        if(param == NTParameterState){
            NTState connectionState = [self->_device readState:nil];
            [self->_brainbitDelegate onStateChange:[self->_device readAddress:nil] state:connectionState];
        }
    }];
}

- (void) unsubscribeToParameterChanged{
    [_device subscribeParameterChangedWithSubscriber:nil];
}

- (void) disconnect{
    [_device disconnect];
}

- (void) close{
    [_device subscribeParameterChangedWithSubscriber:nil];
    [self disconnect];
    
    for (NTResistanceChannel* channel in _resistanceChannels.allValues) {
        [channel subscribeLengthChangedWithSubscribe:nil];
    }
    [_resistanceChannels removeAllObjects];
    
    for (NTSignalChannel* channel in _signalChannels.allValues) {
        [channel subscribeLengthChangedWithSubscribe:nil];
    }
    [_signalChannels removeAllObjects];
    
    [_batteryChannel subscribeLengthChangedWithSubscribe:nil];
    _batteryChannel = nil;
    
    _device = nil;
}

- (NTFirmwareVersion*) version{
    return [_device readFirmwareVersion:nil];
}

- (NSArray*) channels{
    return [_device channels];
}

- (NSArray*) commands{
    return [_device commands];
}

- (NSArray*) parameters{
    return [_device parameters];
}


- (void) initBattery{
    _batteryChannel = [[NTBatteryChannel alloc] initWithDevice:_device];
    [_batteryChannel subscribeLengthChangedWithSubscribe:^(NSUInteger length) {
        [self->_brainbitDelegate onLengthChanged:NTChannelTypeBattery info:[self->_batteryChannel info] length:length];
    }];
}

- (NSArray<NSNumber*>*) readBatteryData: (NSUInteger)offset length:(NSUInteger) length{
    NSArray<NSNumber*>* power = [self->_batteryChannel readDataWithOffset:offset length:length];
    return power;
}
- (NSUInteger) batteryTotalLength{
    return [self->_batteryChannel totalLength];
}

- (void) destroyBattery{
    [_batteryChannel subscribeLengthChangedWithSubscribe:nil];
    _batteryChannel = nil;
}

- (void) startResist{
    [_device executeWithCommand:NTCommandStartResist];
}

- (void) stopResist{
    [_device executeWithCommand:NTCommandStopResist];
}

- (void) initResistanceChannelWithName:(NSString*)channelName{
    NSArray<NTChannelInfo*>* channels = [NTDeviceTraits GetChannelInfoArrayWithType:_device channelType:NTChannelTypeResistance error:nil];
    for (NTChannelInfo* info in channels) {
        if([info.name isEqual:channelName]){
            NTResistanceChannel* channel = [[NTResistanceChannel alloc] initWithDevice:_device channelInfo:info];
            [channel subscribeLengthChangedWithSubscribe:^(NSUInteger length){
                [self->_brainbitDelegate onLengthChanged:channel.info.type info:channel.info length:length];
                
            }];
            [_resistanceChannels setObject:channel forKey:channel.info.name];
        }
    }
}


- (NSArray<NSNumber*>*) readResistanceDataFromChannel: (NSString*)channelName offset:(NSUInteger)offset length:(NSUInteger) length{
    NTResistanceChannel* channel = [_resistanceChannels objectForKey:channelName];
    NSArray<NSNumber*>* data = [channel readDataWithOffset:offset length:length];
    return data;
}

- (NSUInteger) resistanceTotalLengthOfChannel: (NSString*)channelName{
    NTResistanceChannel* channel = [_resistanceChannels objectForKey:channelName];
    return [channel totalLength];
}

- (void) destroyResistanceWithName:(NSString*)channelName{
    NTResistanceChannel* channel = [_resistanceChannels objectForKey:channelName];
    [channel subscribeLengthChangedWithSubscribe:nil];
    channel = nil;
    [_resistanceChannels removeObjectForKey:channelName];
}

- (void) startSignal{
    [_device executeWithCommand:NTCommandStartSignal];
}

- (void) stopSignal{
    [_device executeWithCommand:NTCommandStopSignal];
}

- (void) initSignalWithName:(NSString*)channelName{
    NSArray<NTChannelInfo*>* channels = [NTDeviceTraits GetChannelInfoArrayWithType:_device channelType:NTChannelTypeSignal error:nil];
    for (NTChannelInfo* info in channels) {
        if([info.name isEqual:channelName]){
            NTSignalChannel* channel = [[NTSignalChannel alloc] initWithDevice:_device channelInfo:info];
            [channel subscribeLengthChangedWithSubscribe:^(NSUInteger length){
                [self->_brainbitDelegate onLengthChanged:channel.info.type info:channel.info length:length];
                
            }];
            [_signalChannels setObject:channel forKey:channel.info.name];
        }
    }
}

- (NSArray<NSNumber*>*) readSignalDataFromChannel: (NSString*)channelName offset:(NSUInteger)offset length:(NSUInteger) length{
    NTSignalChannel* channel = [_signalChannels objectForKey:channelName];
    NSArray<NSNumber*>* data = [channel readDataWithOffset:offset length:length];
    return data;
}

- (NSUInteger) signalTotalLengthOfChannel: (NSString*)channelName{
    NTSignalChannel* channel = [_signalChannels objectForKey:channelName];
    return [channel totalLength];
}

- (void) destroySignalWithName:(NSString*)channelName{
    NTSignalChannel* channel = [_signalChannels objectForKey:channelName];
    [channel subscribeLengthChangedWithSubscribe:nil];
    channel = nil;
    [_signalChannels removeObjectForKey:channelName];
}





@end

